import 'dart:convert';
import 'dart:io';

import 'package:path/path.dart';

const _red = '\x1B[31m';
const _green = '\x1B[32m';
const _yellow = '\x1b[33m';
const _reset = '\x1B[0m';
void printSuccess(String message) {
  print('$_green(SUCCESS)$_reset $message');
}

void printFailure(String message) {
  print('$_red(FAILURE)$_reset $message');
}

void printInfo(String message) {
  print('$_yellow(INFO)$_reset $message');
}

Future<void> main(List<String> arguments) async {
  final projectDirectory = _getProjectDirectory();

  final flutterVersion = await _getFlutterVersion();

  await _updateSimpleAppDependencies(projectDirectory: projectDirectory);

  final iOSSDKVersion = await _getIosSDKVersion(
    projectDirectory: projectDirectory,
  );
  final androidSDKVersion = await _getAndroidSDKVersion(
    projectDirectory: projectDirectory,
  );
  final flutterPluginVersion = await _getFlutterPluginVersion(
    projectDirectory: projectDirectory,
  );
  final gitBranch = await _getGitBranch();

  await _saveInfoToAsset(
    projectDirectory: projectDirectory,
    iOSSDKVersion: iOSSDKVersion,
    androidSDKVersion: androidSDKVersion,
    flutterPluginVersion: flutterPluginVersion,
    flutterVersion: flutterVersion,
    gitBranch: gitBranch,
  );

  await _updateSimpleAppDependencies(projectDirectory: projectDirectory);
}

Directory _getProjectDirectory() {
  final scriptDirectoryPathFromMelos = Platform.environment['MELOS_ROOT_PATH'];
  if (scriptDirectoryPathFromMelos != null) {
    printSuccess(
      'Found project directory from melos: $scriptDirectoryPathFromMelos',
    );
    return Directory(scriptDirectoryPathFromMelos);
  }

  printInfo(
    'Melos is not running. Trying to find project directory from current directory.',
  );

  /// Get the directory from which the script was run.
  var scriptDirectory = Directory(dirname(Platform.script.path));

  while (scriptDirectory.path != '/' &&
      !scriptDirectory.path.endsWith('dc_cs-flutter-sdk')) {
    scriptDirectory = scriptDirectory.parent;
  }

  if (scriptDirectory.path == '/') {
    printFailure(
      'Expected this script to be located within the "dc_cs-flutter-sdk" folder. Instead this script is in: ${Platform.script.path}',
    );
    exit(1);
  }

  printSuccess(
      'Found project directory from reccursion: ${scriptDirectory.path}');
  return scriptDirectory;
}

const simpleAppPathRelative = 'packages/contentsquare/example/simple_app';

Future<void> _updateSimpleAppDependencies(
    {required Directory projectDirectory}) async {
  final simpleAppPathAbsolute = join(
    projectDirectory.path,
    simpleAppPathRelative,
  );

  if (!Directory(simpleAppPathAbsolute).existsSync()) {
    printFailure(
      'Could not locate the "simple_app" folder at: $simpleAppPathAbsolute',
    );
    exit(1);
  }
  final result = await Process.run(
    'melos',
    [
      'exec',
      '--scope="*example*"',
      '--',
      'flutter',
      'pub',
      'get',
      simpleAppPathAbsolute
    ],
    runInShell: true,
  );

  if (result.exitCode != 0) {
    printFailure(
      'Could not update the "simple_app" dependencies.\n'
      'stdout:\n'
      '${result.stdout}\n'
      'stderr:\n'
      '${result.stderr}',
    );
    exit(1);
  } else {
    printSuccess('Updated the "simple_app" dependencies.');
  }
}

const contentsquarePackagePathRelative = 'packages/contentsquare';
const podspecWithSDKPathRelative =
    contentsquarePackagePathRelative + '/ios/contentsquare.podspec';
final contentsquareIOSSDKRegex = RegExp(
  r'''
  s.dependency 'CS_iOS_SDK', '([\d|.]*?)'
''',
);

Future<String> _getIosSDKVersion({required Directory projectDirectory}) async {
  final podspecWithSDKPathAbsolute = join(
    projectDirectory.path,
    podspecWithSDKPathRelative,
  );

  // Read the podspec file.
  final podspecFile = File(podspecWithSDKPathAbsolute);
  if (!await podspecFile.exists()) {
    printFailure(
        'Could not find contentsquare.podspec file at: $podspecWithSDKPathAbsolute',);
    exit(1);
  }

  final podspecContents = await podspecFile.readAsString();
  final contentsquareIOSSDKMatch =
      contentsquareIOSSDKRegex.firstMatch(podspecContents);
  if (contentsquareIOSSDKMatch == null) {
    printFailure(
      'Could not find the iOS SDK version in the contentsquare.podspec file: Regexp match failed.\n'
      "DO make sure to use restricting versionning: (e.g. \"s.dependency 'CS_iOS_SDK', '1.0.0'\")",
    );
    exit(1);
  }

  final version = contentsquareIOSSDKMatch.group(1)!;

  printSuccess('Found iOS SDK version: $version');

  return version;
}

const gradleWithSDKPathRelative =
    contentsquarePackagePathRelative + '/android/build.gradle';
final contentsquareAndroidSDKRegex =
    RegExp(r'"com.contentsquare.android:library:([\d|.]*)"');

Future<String> _getAndroidSDKVersion({
  required Directory projectDirectory,
}) async {
  final gradleWithSDKPathAbsolute = join(
    projectDirectory.path,
    gradleWithSDKPathRelative,
  );

  // Read the podfileLock file.
  final gradleFile = File(gradleWithSDKPathAbsolute);
  if (!await gradleFile.exists()) {
    printFailure('Could not find gradle file at: $gradleWithSDKPathAbsolute');
    exit(1);
  }

  final gradleContents = await gradleFile.readAsString();
  final contentsquareAndroidSDKMatch =
      contentsquareAndroidSDKRegex.firstMatch(gradleContents);
  if (contentsquareAndroidSDKMatch == null) {
    printFailure(
      'Could not find the Android SDK version in the gradle file: Regexp match failed.',
    );
    exit(1);
  }

  final version = contentsquareAndroidSDKMatch.group(1)!;

  printSuccess('Found Android SDK version: $version');

  return version;
}

const pubspecLockWithPluginPathRelative =
    simpleAppPathRelative + '/pubspec.lock';
final contentsquareFlutterPluginRegex = RegExp(
  r'''
  contentsquare:
    .*?
    version: "([\d|.]*?)"
''',
  multiLine: true,
  dotAll: true,
);

Future<String> _getFlutterPluginVersion({
  required Directory projectDirectory,
}) async {
  final pubspecLockWithPluginPathAbsolute = join(
    projectDirectory.path,
    pubspecLockWithPluginPathRelative,
  );

  // Read the pubspec file.
  final pubspecLockFile = File(pubspecLockWithPluginPathAbsolute);
  if (!await pubspecLockFile.exists()) {
    printFailure(
        'Could not find pubspec.lock file at: $pubspecLockWithPluginPathAbsolute');
    exit(1);
  }

  final pubspecLockContents = await pubspecLockFile.readAsString();
  final contentsquareFlutterPluginMatch =
      contentsquareFlutterPluginRegex.firstMatch(pubspecLockContents);
  if (contentsquareFlutterPluginMatch == null) {
    printFailure(
      'Could not find the Flutter plugin version in the pubspec.lock file: Regexp match failed.',
    );
    exit(1);
  }

  final version = contentsquareFlutterPluginMatch.group(1)!;

  printSuccess('Found Flutter plugin version: $version');

  return version;
}

final flutterVersionRegex = RegExp(r'Flutter ([\d|.]*?) •');

Future<String> _getFlutterVersion() async {
  final result = await Process.run(
    'flutter',
    ['--version'],
    runInShell: true,
  );

  if (result.exitCode != 0) {
    printFailure(
      'Could not get the Flutter version in the output of "flutter --version".\n'
      'stdout:\n'
      '${result.stdout}\n'
      'stderr:\n'
      '${result.stderr}',
    );
    exit(1);
  }

  final flutterVersionOutput = result.stdout;
  final flutterVersionMatch =
      flutterVersionRegex.firstMatch(flutterVersionOutput);
  if (flutterVersionMatch == null) {
    printFailure(
      'Could not find the Flutter version in the output of "flutter --version": Regexp match failed.',
    );
    exit(1);
  }

  final version = flutterVersionMatch.group(1)!;

  printSuccess('Found Flutter version: $version');

  return version;
}

Future<String> _getGitBranch() async {
  final result =
      await Process.run('git', ['rev-parse', '--abbrev-ref', 'HEAD']);

  if (result.exitCode != 0) {
    printFailure(
      'Could not get the current git branch.\n'
      'stdout:\n'
      '${result.stdout}\n'
      'stderr:\n'
      '${result.stderr}',
    );
    exit(1);
  }

  final branch = result.stdout.trim();

  printSuccess('Found git branch: $branch');

  return branch;
}

const versionAssetPathRelative =
    simpleAppPathRelative + '/assets/versions_info.json';

Future<void> _saveInfoToAsset({
  required Directory projectDirectory,
  required String iOSSDKVersion,
  required String androidSDKVersion,
  required String flutterPluginVersion,
  required String flutterVersion,
  required String gitBranch,
}) async {
  final versionAssetPathAbsolute = join(
    projectDirectory.path,
    versionAssetPathRelative,
  );

  final versionAssetFile = File(versionAssetPathAbsolute);
  if (!await versionAssetFile.exists()) {
    await versionAssetFile.create(recursive: true);
  }

  await versionAssetFile.writeAsString(
    jsonEncode(
      {
        'iOSSDKVersion': iOSSDKVersion,
        'androidSDKVersion': androidSDKVersion,
        'flutterPluginVersion': flutterPluginVersion,
        'flutterVersion': flutterVersion,
        'gitBranch': gitBranch,
      },
    ),
  );

  printSuccess('Saved version info to asset: $versionAssetPathAbsolute');
}
